import { Plugin } from 'ckeditor5/src/core';
import { Widget, toWidget, viewToModelPositionOutsideModelElement } from 'ckeditor5/src/widget';
import InlineRelatedPostCommand from './InlineRelatedPostCommand';

export default class InlineRelatedPostEditing extends Plugin {
  static get requires() {                                                    // ADDED
    return [ Widget ];
  }
  init() {
    console.log( 'InlineRelatedPostEditing#init() got called' );

    this._defineSchema();
    this._defineConverters();

    this.editor.commands.add( 'inline-related-post', new InlineRelatedPostCommand( this.editor ) );

    this.editor.editing.mapper.on(
      'viewToModelPosition',
      viewToModelPositionOutsideModelElement( this.editor.model, viewElement => viewElement.hasClass( 'inline-related-post' ) )
    );
  }

  _defineSchema() {
    const schema = this.editor.model.schema;
    schema.register( 'inline-related-post', {
      // Behaves like a self-contained inline object (e.g. an inline image)
      // allowed in places where $text is allowed (e.g. in paragraphs).
      // The inline widget can have the same attributes as text (for example linkHref, bold).
      inheritAllFrom: '$inlineObject',

      // The placeholder can have many types, like date, name, surname, etc:
      allowAttributes: [ 'name' ]
    } );
  }

  _defineConverters() {                                                      // ADDED
    const conversion = this.editor.conversion;

    conversion.for( 'upcast' ).elementToElement( {
      view: {
        name: 'slot',
        classes: [ 'inline-related-post' ]
      },
      model: ( viewElement, { writer: modelWriter } ) => {
        // Extract the "name" from "{name}".
        const name = viewElement.getChild( 0 ).data.slice( 1, -1 );

        return modelWriter.createElement( 'inline-related-post', { name } );
      }
    } );

    conversion.for( 'editingDowncast' ).elementToElement( {
      model: 'inline-related-post',
      view: ( modelItem, { writer: viewWriter } ) => {
        const widgetElement = createInlineRelatedPostView( modelItem, viewWriter );

        // Enable widget handling on a inline-related-post element inside the editing view.
        return toWidget( widgetElement, viewWriter );
      }
    } );

    conversion.for( 'dataDowncast' ).elementToElement( {
      model: 'inline-related-post',
      view: ( modelItem, { writer: viewWriter } ) => createInlineRelatedPostView( modelItem, viewWriter )
    } );

    // Helper method for both downcast converters.
    function createInlineRelatedPostView( modelItem, viewWriter ) {
      const name = modelItem.getAttribute( 'name' );

      const inlineRelatedPostView = viewWriter.createContainerElement( 'slot', {
        class: 'inline-related-post'
      } );

      // Insert the inline-related-postname (as a text).
      const innerText = viewWriter.createText( '{' + name + '}' );
      viewWriter.insert( viewWriter.createPositionAt( inlineRelatedPostView, 0 ), innerText );

      return inlineRelatedPostView;
    }
  }

}

