import { Plugin } from 'ckeditor5/src/core';
import { Widget, toWidget, viewToModelPositionOutsideModelElement } from 'ckeditor5/src/widget';
import InlineRelatedPostCommand from './InlineRelatedPostCommand';

export default class InlineRelatedPostEditing extends Plugin {
  static get requires() {
    return [ Widget ];
  }
  init() {

    this._defineSchema();
    this._defineConverters();

    this.editor.commands.add( 'inline-related-post', new InlineRelatedPostCommand( this.editor ) );

    this.editor.editing.mapper.on(
      'viewToModelPosition',
      viewToModelPositionOutsideModelElement( this.editor.model, viewElement => viewElement.hasClass( 'inline-related-post' ) )
    );
  }

  _defineSchema() {
    const schema = this.editor.model.schema;
    schema.register( 'inline-related-post', {
      // Behaves like a self-contained inline object (e.g. an inline image)
      // allowed in places where $text is allowed (e.g. in paragraphs).
      // The inline widget can have the same attributes as text (for example linkHref, bold).
      //inheritAllFrom: '$inlineObject',
      //
      // Behaves like a self-contained block object (e.g. a block image)
      // allowed in places where other blocks are allowed (e.g. directly in the root).
      inheritAllFrom: '$blockObject',

      // The placeholder can have many types, like date, name, surname, etc:
      allowAttributes: [ 'name',  'data-related-post-id', 'data-drupal-field', 'data-related-url' ]
    } );
  }

  _defineConverters() {                                                      // ADDED
    const conversion = this.editor.conversion;

    conversion.for( 'upcast' ).elementToElement( {
      view: {
        name: 'slot',
        classes: [ 'inline-related-post' ],
        attributes: [
          'data-related-post-id',
          'data-drupal-field',
          'data-related-url'
        ]
      },
      model: ( viewElement, { writer: modelWriter } ) => {
        // Extract the "name" from "{slot-related-post-id:name}".
        //const name = viewElement.getChild( 0 ).data.slice( 22, -1 );
        const name = viewElement.getAttribute('data-related-post-id');

        return modelWriter.createElement( 'inline-related-post', {
          name,
          'data-related-post-id': viewElement.getAttribute('data-related-post-id'),
          'data-related-url': viewElement.getAttribute('data-related-url'),
          'data-drupal-field': viewElement.getAttribute('data-drupal-field')

        } );
      }
    } );

    conversion.for( 'editingDowncast' ).elementToElement( {
      model: 'inline-related-post',
      view: ( modelItem, { writer: viewWriter } ) => {
        const widgetElement = createInlineRelatedPostView( modelItem, viewWriter, true );

        // Enable widget handling on a inline-related-post element inside the editing view.
        return toWidget( widgetElement, viewWriter );
      }
    } );

    conversion.for( 'dataDowncast' ).elementToElement( {
      model: 'inline-related-post',
      view: ( modelItem, { writer: viewWriter } ) => createInlineRelatedPostView( modelItem, viewWriter, false )
    } );

    // Helper method for both downcast converters.
    function createInlineRelatedPostView( modelItem, viewWriter, info ) {
      const name = modelItem.getAttribute( 'data-related-post-id' );
      const url = modelItem.getAttribute( 'data-related-url' );
      const drupalField = modelItem.getAttribute('data-drupal-field');

      const inlineRelatedPostView = viewWriter.createContainerElement( 'slot', {
        class: 'inline-related-post',
        'data-related-post-id': name,
        'data-drupal-field': drupalField,
        'data-related-url': url
      } );

      // Insert the inline-related-postname (as a text).
      if (info) {
      const innerText = viewWriter.createText( `{slot-related-post-id:${name} (url:${url})}` );
        viewWriter.insert( viewWriter.createPositionAt( inlineRelatedPostView, 0 ), innerText );
      }

      return inlineRelatedPostView;
    }
  }

}

