<?php

declare(strict_types=1);

namespace Drupal\inline_related_posts\Hook;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Site\Settings;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Component\Utility\Html;
use Drupal\Core\Url;

/**
 * Contains Theme hooks - class is declared as a service in services.yml file.
 *
 * @see https://drupalize.me/blog/drupal-111-adds-hooks-classes-history-how-and-tutorials-weve-updated
 */
final class InlineRelatedPostsThemeHooks {

  /**
   * Constructs an InlineRelatedPostsThemeHooks object.
   */
  public function __construct(
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly Settings $settings,
  ) {}

  /**
   * Implements hook_preprocess_HOOK().
   *
   * Note: Hook preprocess_node must remain procedural (message in D11.1).
   * in Drupal\Core\Hook\HookCollectorPass::checkForProceduralOnlyHooks().
   */
  public function preprocessNode(&$variables) {
    // @todo move config to autodetect from text from ckeditor or edited
    // from texts format (the field origin of related links), and text
    // type fields settings...
    $config = [
      // The content bundle that allow inline related in its body.
      'bundle_host' => $variables['related_posts_bundle_host'] ?: 'articles',
      // The host view mode where to add inline related content.
      'host_view_mode' => $variables['related_posts_bundle_host_view_mode'] ?: 'full',
      // The view-mode used to render the related content.
      'inline_view_mode' => $variables['related_posts_inline_view_mode'] ?: 'inline_related',
      // The length of paragraph to be surpassed to append inline related post.
      'paragraph_length' => $variables['related_posts_paragraph_length'] ?: 1000,
      // The field that holds the related links in host entity.
      'host_related_links_field' => $variables['related_posts_host_related_links_field'] ?: 'field_links_relacionats',
    ];
    if ($variables["elements"]["#node"]->{$config['host_related_links_field']}) {
      /* Inline related insert */
      if ($variables["elements"]["#node"]->bundle() === $config['bundle_host'] && $variables["view_mode"] === $config['host_view_mode']) {
        // Return earlier if not link type.
        // @todo allow entity reference too.
        if ($variables["elements"]["#node"]->{$config['host_related_links_field']} &&
        $variables["elements"]["#node"]->{$config['host_related_links_field']}->getFieldDefinition()->getType() !== 'link') {
          return;
        }

        $view_builder = $this->entityTypeManager->getViewBuilder("node");

        $related_links = [];
        /* Load html */
        foreach ($variables["elements"]["#node"]->{$config['host_related_links_field']} as $related_links_item) {
          // Check if field is of link type.
          // start defining the related_url (with possible overwrite if local)
          $related_url_string = $related_links_item->getValue()['uri'];
          if (UrlHelper::isExternal($related_links_item->getValue()['uri'])) {
            $trusted_host_patterns = implode('|', Settings::get('trusted_host_patterns', []));
            $parsed_url = parse_url($related_links_item->getValue()['uri']);
            if (preg_match("/$trusted_host_patterns/", $parsed_url['host'])) {
              // The external in fact is local ...
              $related_url_string = $parsed_url['path'];
              $related_url = Url::fromUserInput($related_url_string);
            }
            else {
              // The external is no local.
              $related_url = Url::fromUri($related_url_string);
            }
          }
          else {
            // The link is really local.
            $related_url = Url::fromUserInput($related_url_string);
          }
          if ($related_url->isRouted() && $related_url->getRouteName() == 'entity.node.canonical') {
            $related_node = $this->entityTypeManager->getStorage('node')->load($related_url->getRouteParameters()['node']);
            // Current language?, 'ca');.
            $related_link = $view_builder->view($related_node, $config['inline_view_mode']);
          }
          if (!$related_url->isRouted()) {
            $related_link = [
              '#type' => 'link',
              '#title' => $related_links_item->getValue()['title'],
              '#url' => $related_url,
              '#attributes' => [
                'class' => ['related-link-internal'],
              ],
            ];
          }
          $related_links[] = [
            '#type' => 'container',
            '#attributes' => [
              'class' => ['related-link'],
            ],
            'link' => $related_link,
          ];
        }
        // Return earlier if no related_links.
        if (empty($related_links)) {
          return;
        }

        /* saving body to reuse all but text properties template */
        $complete_body = $variables['content']['body'][0];
        $dom = Html::load($complete_body['#text']);

        $prev_text_size = 0;
        $prev_text_contents = [];
        /** @var counter $body_paragraphs to readd each paragraph with
         *  related links after in body */
        $body_paragraph_id = 0;
        $inline_related_id = 0;

        // First try to detect slots placed manually there by ckeditor.
        $manual_slots = FALSE;
        foreach ($dom->getElementsByTagName('slot') as $slot) {
          if ($slot->nodeName === 'slot' && $slot->getAttribute('class') === "inline-related-post" && strlen($slot->getAttribute('data-related-post-id')) > 0) {
            // Trigger manual_mode flag and break:it is placed using ckeditor.
            $manual_slots = TRUE;
            break;
          }
        }

        // Manual mode iterating to find slots.
        foreach ($dom->getElementsByTagName('body')->item(0)->childNodes as $p) {
          if ($manual_slots) {
            if ($p->nodeName === 'slot' && $p->getAttribute('class') === "inline-related-post" && strlen($p->getAttribute('data-related-post-id')) > 0) {
              $inline_related_id = $p->getAttribute('data-related-post-id');
              // Add inline related link.
              $variables['content']['body'][$body_paragraph_id] = $related_links[$inline_related_id];
              $body_paragraph_id++;
              // Remove from related_ids to add the remaining later.
              unset($related_links[$inline_related_id]);
            }
            else {
              $text_content = $dom->saveHtml($p);
              // Clone body to copy properties and add text after.
              $variables['content']['body'][$body_paragraph_id] = $complete_body;
              $variables['content']['body'][$body_paragraph_id]['#text'] = $text_content;
              $body_paragraph_id++;
            }
          }
          else {
            // Automatic behavior.
            if (strlen($p->textContent) + $prev_text_size < $config['paragraph_length']) {
              $prev_text_size = strlen($p->textContent);
              $prev_text_contents[] = $dom->saveHtml($p);
              continue;
            }
            $text_content = implode(separator:"", array:$prev_text_contents) . $dom->saveHtml($p);
            $prev_text_size = 0;
            // Reset previous text contents.
            $prev_text_contents = [];
            // Clone body to copy properties and add text after.
            $variables['content']['body'][$body_paragraph_id] = $complete_body;
            $variables['content']['body'][$body_paragraph_id]['#text'] = $text_content;
            $body_paragraph_id++;
            // Add inline just after text.
            $variables['content']['body'][$body_paragraph_id] = $related_links[$inline_related_id];
            $body_paragraph_id++;
            unset($related_links[$inline_related_id]);
            $inline_related_id++;
          }
        }

        // @todo Add the rest related_links if not placed $related_links.
        // Easy to be done as $related_links now has been trimmed.
        $variables['inline_related_posts_remaining'] = $related_links;
      }
    }
  }

}
